/* This file is a component of the JunkBusterMT Proxy, based on original code from
 * the Internet Junkbuster Proxy.
 *
 * Copyright (C) 2000 Gabriel L. Somlo
 * Copyright (C) 1997 Anonymous Coders and Junkbusters Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see ../GPL.txt); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <pthread.h>
#include <stdio.h>
#include <string.h>


static int stats_enabled = 0;

static int             urls_passed   = 0;
static int             urls_rejected = 0;
static pthread_mutex_t stats_mutex;


/*-----------------------------------------------------------------*/
/* Call this from main() before multithreaded service cycle starts */
/*-----------------------------------------------------------------*/
void
init_stats( int include_stats ) {

  pthread_mutex_init( &stats_mutex, NULL );
  stats_enabled = include_stats;

} /* init_stats */


/*--------------------------------------------------------------*/
/* Call this from chat() after we're sure we didn't get blocked */
/*--------------------------------------------------------------*/
void
inc_urls_passed( void ) {

  pthread_mutex_lock( &stats_mutex );
  ++urls_passed;
  pthread_mutex_unlock( &stats_mutex );

} /* inc_urls_passed */


/*-----------------------------------------------------------*/
/* Call this from chat() after we're sure we did get blocked */
/*-----------------------------------------------------------*/
void
inc_urls_rejected( void ) {

  pthread_mutex_lock( &stats_mutex );
  ++urls_rejected;
  pthread_mutex_unlock( &stats_mutex );

} /* inc_urls_rejected */


/*-----------------------------------------------*/
/* Call this from the handler of show-proxy-args */
/*-----------------------------------------------*/
char *
get_url_stats( void ) {

  char buf[BUFSIZ];
  int passed, rejected, total;

  if( !stats_enabled )
    return( NULL );

  /* make local copies and get out of the way as fast as possible */
  pthread_mutex_lock( &stats_mutex );
  passed = urls_passed;
  rejected = urls_rejected;
  pthread_mutex_unlock( &stats_mutex );

  if( (total = passed + rejected) > 0 )
    sprintf( buf, "<h2>Block statistics are as follows:</h2>\n"
                  "%d total requests, %d (%6.2f%%) passed, %d (%6.2f%%) "
                  "blocked.<br>\n",
                  total,
                  passed,   (double)passed   * 100.0 / (double)total,
                  rejected, (double)rejected * 100.0 / (double)total );
  else
    sprintf( buf, "<h2>No block statistics available "
                  "(no activity recorded so far)</h2></br>\n" );

  return( strdup(buf) );

} /* get_url_stats */
