/* This file is a component of the JunkBusterMT Proxy, based on original code from
 * the Internet Junkbuster Proxy.
 *
 * Copyright (C) 2000 Gabriel L. Somlo
 * Copyright (C) 1997 Anonymous Coders and Junkbusters Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see ../GPL.txt); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>

#include "errlog.h"
#include "miscstr.h"

#include "gnu_regex.h"

#include "urlcodec.h"

#include "smrtcook.h"

#include "chat.h"

#include "filegard.h"

#include "parsers.h"

#include "filters.h"

#include "bindconn.h"

#include "csp.h"

#include "sed.h"

#include "blckstat.h"


char CFAIL[] =
  "HTTP/1.0 503 Connect failed\n"
  "Content-Type: text/html\n\n"
  "<html>\n"
  "<head>\n"
  "<title>Internet Junkbuster: Connect failed</title>\n"
  "</head>\n"
  BODY
  "<h1><center>"
  BANNER
  "</center></h1>"
  "TCP connection to '%s' failed: %s.\n<br>\n"
  "This means that the destination site is down or unreachable. You may want "
  "to try again in a few minutes...\n"
  "</body>\n"
  "</html>\n"
  ;

char CNXDOM[]  = "HTTP/1.0 404 Non-existent domain\n"
		 "Content-Type: text/html\n\n"
		 "<html>\n"
		 "<head>\n"
		 "<title>Internet Junkbuster: Non-existent domain</title>\n"
		 "</head>\n"
		 BODY
		 "<h1><center>"
		 BANNER
		 "</center></h1>"
		 "No such domain: %s\n"
		 "</body>\n"
		 "</html>\n"
		 ;

char CSUCCEED[] = "HTTP/1.0 200 Connection established\n"
		  "Proxy-Agent: IJ/" VERSION "\n\n"
		  ;

char CPROXYAUTH[] =
  "HTTP/1.1 407 Proxy Authentication Required\n"
  "Proxy-Authenticate: Basic realm=\"SurfAgent\"\n"
  "Content-Type: text/html\n\n"
  "<html>\n"
  "<head>\n"
  BODY
  "<h1><center>"
  BANNER ": 407 Proxy Authentication Required"
  "</center></h1>"
  "In order to take full advantage of " BANNER " you need an "
  "account (login-password) combination on the proxy. You can create a new "
  "account by simply entering a new login-password combination, and "
  BANNER " will start learning your profile from scratch.<p>\n"
  "Currently, " BANNER " does not keep track of the identity of "
  "its users. In other words, the owner of the agent has no idea who you "
  "are. Your secrets are safe with us...\n"
  "</body>\n"
  "</html>\n"
  ;

char CFEEDBACK[] =
  "</p><hr>\n"
  "<center><table border bgcolor=" BGCOLOR "><tr><td>\n"
  /* "target=\"bogus_frame\" " */
  "<a href=\"http://SurfAgent/sa-feedback-url?%s!%s!%s\">"
  BANNER
  /* " <font color=" LNKCOLOR ">Feedback</font>" */
  "</a>\n"
  "</td></tr></table></center>\n"
  ;

char CHEADER[] = "HTTP/1.0 400 Invalid header received from browser\n\n";

char SHEADER[] = "HTTP/1.0 502 Invalid header received from server\n\n";

char VANILLA_WAFER[] =
	"NOTICE=TO_WHOM_IT_MAY_CONCERN_"
	"Do_not_send_me_any_copyrighted_information_other_than_the_"
	"document_that_I_am_requesting_or_any_of_its_necessary_components._"
	"In_particular_do_not_send_me_any_cookies_that_"
	"are_subject_to_a_claim_of_copyright_by_anybody._"
	"Take_notice_that_I_refuse_to_be_bound_by_any_license_condition_"
	"(copyright_or_otherwise)_applying_to_any_cookie._";


char BLANKGIF[] = "HTTP/1.0 200 OK\r\n" 
                  "Content-type: image/gif\r\n\r\n" 
                  "GIF89a\001\000\001\000\200\000\000\377\377\377\000\000" 
                  "\000!\371\004\001\000\000\000\000,\000\000\000\000\001" 
                  "\000\001\000\000\002\002D\001\000;"; 

char JBGIF[] = "HTTP/1.0 200 OK\r\n"
                  "Content-type: image/gif\r\n\r\n"
                  "GIF89aD\000\013\000\360\000\000\000\000\000\377\377\377!"
                  "\371\004\001\000\000\001\000,\000\000\000\000D\000\013\000"
                  "\000\002a\214\217\251\313\355\277\000\200G&K\025\316hC\037"
                  "\200\234\230Y\2309\235S\230\266\206\372J\253<\3131\253\271"
                  "\270\215\342\254\013\203\371\202\264\334P\207\332\020o\266"
                  "N\215I\332=\211\312\3513\266:\026AK)\364\370\365aobr\305"
                  "\372\003S\275\274k2\354\254z\347?\335\274x\306^9\374\276"
                  "\037Q\000\000;";





int tinygif = 1;		/* controls replacement of blocked images */



/*--------------------------------------------------------------------*/
/* read, process, and send the client header to the server            */
/* also send the start of the client body - whatever came in the same */
/*   buffer with the end of the header...                             */
/* once the connection is set up, we pass data                        */
/*  back and forth between the client and the server until done       */
/* this function also processes the server header                     */
/*--------------------------------------------------------------------*/
void
chat( struct client_state *csp ) {

  fd_set rfds;
  int maxfd, n, leftoversz;
  char buf[BUFSIZ], *hdr, *leftover, *p;
  struct http_request *http = csp->http;

  /* read the client request (first client header line) */
  /*  note: since we're not using select() we could get blocked if a client
   *        connected and then didn't say anything !!
   */
  for( ; ; ) {
    if( (n = read_socket(csp->cfd, buf, BUFSIZ)) <= 0 )
      break;		/* error! */

    add_to_iob( csp->iob, buf, n );

    if( !(p = get_next_header_from_iob(csp->iob)) )
      break;		/* no HTTP request! */

    if( *p == '\0' )
      continue;		/* more to come! (iob does not yet have a full line) */

    parse_http_request( p, http );
    free( p );
    break;
  } /* for( ; ; ) - done reading request from the client */

/*
  show_http_request( http );
*/

  if( !(http->cmd) ) { /* wrong header from client, bitch about it... */
    strcpy( buf, CHEADER );
    write_and_log( csp->cfd, buf, strlen(buf) );
    return;
  }

  /* decide how to route the HTTP request */
  if( !(csp->gw = forward_url(http)) ) {
    errlogf( ERR, "chat(): gateway spec is NULL!?!? This can't happen!\n" );
    exit( 1 );
  }

  /* decide on the final form of the client request header line */
  if( csp->gw->forward_host )
    /* forwarding, just pass the original request as is */
    enlist( csp->headers, http->cmd );
  else
    /* if we're doing an ssl transaction, the request gets dropped */
    if( !(http->ssl) ) {
      /* otherwise re-create request with path-only (no http://domainspec) */
      sprintf( buf, "%s %s %s", http->gpc, http->path, http->ver );
      enlist( csp->headers, buf );
    }

  /* grab the rest of the client's headers */
  for( ; ; ) {
    if( !(p = get_next_header_from_iob(csp->iob)) )
      break;

    if( *p == '\0' ) {
      if( (n = read_socket(csp->cfd, buf, BUFSIZ)) <= 0 ) {
        errlogf( ERN, "chat(): read from client failed\n" );
        return;
      }
      add_to_iob( csp->iob, buf, n );
      continue;
    }

    enlist( csp->headers, p );
    free( p );
  } /* for( ; ; ) - done grabbing rest of client's headers */

  /* decide what to do with cookies (needed before sed() call) */
  csp->cookie_status = cookie_url( http );

  /* filter client headers and put result into header string hdr */
  hdr = sed( SED_CLI, csp );

  /* check for URLs on the blocklist */
  /* csp->accept_image_only - parsed by sed() - is used here... */
  if( !(http->noblock) && (p = block_url(http)) ) {
    errlogf( GPC, "%s%s crunch!\n", http->hostport, http->path );

    if( csp->accept_image_only )
      switch( tinygif ) {
        case 1:
          write_and_log( csp->cfd, BLANKGIF, sizeof(BLANKGIF) - 1 );
          break;
        case 2:
          write_and_log( csp->cfd, JBGIF, sizeof(JBGIF) - 1 );
          break;
        default:	/* case 0: and any possible error */
          write_and_log( csp->cfd, p, strlen(p) );
          break;
      } /* switch( tinygif ) */
    else /* csp->accept_image_only is false */
      write_and_log( csp->cfd, p, strlen(p) );

    freez( p );
    freez( hdr );

    /* increment number of rejected requests */
    inc_urls_rejected();

    return;
  } /* if( ... block_url ... ) */

  /* check for "fake", JB-created URLs */
  if( (p = intercept_url(csp)) ) {
    errlogf( GPC, "%s%s crunch!\n", http->hostport, http->path );
    write_and_log( csp->cfd, p, strlen(p) );
    freez( p );
    freez( hdr );

    return;
  } /* if( ... intercept_url ... ) */

  /* hdr is now final, we don't need the raw parsed headers anymore... */
  destroy_list( csp->headers );	

  /* log client request and connection method */
  errlogf( GPC, "%s%s\n", http->hostport, http->path );
  if( csp->gw->forward_host )
    errlogf( CON, "connecting via %s:%d to: %s ... ",
                  csp->gw->forward_host, csp->gw->forward_port,
                  http->hostport );
  else
    errlogf( CON, "connecting to: %s ... ", http->hostport );

  /* increment number of passed requests */
  inc_urls_passed();

  /* here we connect to the server, gateway, or the forwarder */
  if( (csp->sfd = (csp->gw->conn)(csp->gw, http, csp->ip_addr_long)) < 0 ) {
    errlogf( ERN, "chat(): failed to connect to: %s\n", http->hostport );

    if( errno == EINVAL )
      sprintf( buf, CNXDOM, http->host );
    else
      sprintf( buf, CFAIL, http->hostport, strerror(errno) );
    write_and_log( csp->cfd, buf, strlen(buf) );

    freez( hdr );

    return;
  } /* if( csp->sfd... ) - done connecting to serv., gw., or fwdr. */
  errlogf( CON, "OK\n" );

  /* flush the iob buffer */
  leftover = flush_iob( csp->iob, &leftoversz );

  /* send client header */
  if( csp->gw->forward_host || !(http->ssl) ) {
    /* write the client's (modified) header to the server */
    /*  (along with anything else that may be in the buffer) */

    n = strlen( hdr );
    if( (write_and_log(csp->sfd, hdr, n) != n) ||
        (write_and_log(csp->sfd, leftover, leftoversz) != leftoversz)
      ) {
      errlogf( ERN, "chat(): failed writing header to: %s\n", http->hostport );

      sprintf( buf, CFAIL, http->hostport, strerror(errno) );
      write_and_log( csp->cfd, buf, strlen(buf) );

      freez( hdr );
      freez( leftover );

      return;
    } /* if( write_and_log ) - done writing header */

  } else {
    /* we're running an SSL tunnel and we're not forwarding, so just send */
    /*  the "connect succeeded" message to the client, flush the rest, */
    /*  and get out of the way. */

    if( write_and_log(csp->cfd, CSUCCEED, sizeof(CSUCCEED)-1) < 0 ) {
      freez( hdr );
      freez( leftover );
      return;
    }

  } /* done sending client header */

  /* we're finished with the client's header */
  freez( hdr );
  freez( leftover );

  /* Connection to server is now set up, we can proceed passing data back
   *  and forth between the client and the server...
   */

  maxfd = ( csp->cfd > csp->sfd ) ? csp->cfd : csp->sfd;  /* highest fd # */
  csp->server_body = 0;	/* first thing from server will be the header */

  /* pass data between client and server until one shuts down the connection */
  for( ; ; ) {

    FD_ZERO( &rfds );
    FD_SET( csp->cfd, &rfds );
    FD_SET( csp->sfd, &rfds );

    /* wait for activity */
    if( select(maxfd+1, &rfds, NULL, NULL, NULL) < 0 ) {
      errlogf( ERN, "chat(): call to select() failed!\n" );
      break;	/* skip out of for, i.e., return */
    }

    /* handle the body of the client request: simply read/write */
    if( FD_ISSET(csp->cfd, &rfds) ) {
      if( (n = read_socket(csp->cfd, buf, BUFSIZ)) <= 0 )
        break;		/* client gone, return */

      if( write_and_log(csp->sfd, buf, n) != n ) {
        errlogf( ERN, "chat(): failed writing to: %s\n", http->host );
        break;		/* server error, return */
      }

      continue;		/* we're ok, keep going */
    } /* if( FD_ISSET(client fd) ) - handling body of client request */

    /* the server is ready to talk; hdr or body depending on csp->server_body */
    if( FD_ISSET(csp->sfd, &rfds) ) {
      if( (n = read_socket(csp->sfd, buf, BUFSIZ)) < 0 ) {

        /* log the error */
        errlogf( ERN, "chat(): failed reading from: %s\n", http->host );

        /* notify the client */
        sprintf( buf, CFAIL, http->hostport, strerror(errno) );
        write_and_log( csp->cfd, buf, strlen(buf) );

        break;	/* skip out of for, i.e., return */
      } /* done reading from server, now process... */

      /* we're at the end of the downloaded document */
      if( n == 0 ) {

        /* done downloading */
        break;		/* return */

      } /* if( n == 0 ) - done dealing with end-of-the-document stuff */

      /* since n > 0 see if we're in the body or the header */
      if( csp->server_body || http->ssl ) {
        /* we're in the srv. body or we have a ssl conn. ->write to client */

        /* write to client (unless we're analyzing the document for a categ.) */
        if( write_and_log(csp->cfd, buf, n) != n ) {
          errlogf( ERN, "chat(): failed writing to client\n" );
          break;	/* skip out of the for, i.e., return */
        }

      } else {
        /* we're still looking for the end of the server's header */

        /* buffer up the data we just read */
        add_to_iob( csp->iob, buf, n );

        /* get header lines from the iob */
        while( (p = get_next_header_from_iob(csp->iob)) && (*p != '\0') ) {
          enlist( csp->headers, p );
          free( p );
        }

        /* we run out of data in the iob, before we got all the headers; */
        /*  we need to go back and read more; (*p == '\0')               */
        if( p )
          continue;

        /* we have now received the entire header, and we can filter it now */
        hdr = sed( SED_SRV, csp );

        /* flush the iob */
        leftover = flush_iob( csp->iob, &leftoversz );

        /* write header and leftover buffer to the client */
        n = strlen( hdr );
        if( (write_and_log(csp->cfd, hdr, n) != n ) ||
            (write_and_log(csp->cfd, leftover, leftoversz) != leftoversz) ) {

          errlogf( ERN, "chat(): failed writing header to client\n" );

          /* the write failed, so don't bother telling the client... */
          freez( hdr );
          freez( leftover );
          break;	/* skip out of the for, i.e., return */
        }

        /* we're finished with the server's header, now clean up */
        freez( hdr );
        freez( leftover );
        csp->server_body = 1;

      } /* done with server header */

    } /* if( FD_ISSET(server fd) ) - handling server activity */

  } /* for( ; ; ) - done passing stuff between the client and the server */

} /* chat */



/*---------------------------------------------------------------------------*/
/* This wrapper function calls all the functions required to serve a request */
/*  from the browser. The argument csp must point to an allocated structure, */
/*  with the client file descriptor csp->cfd open.                           */
/* Note: In other words, this function is the "main" of each thread...       */
/*---------------------------------------------------------------------------*/
void *
serve( void *arg ) {

  struct client_state *csp;
  
  csp = (struct client_state *)arg;

  chat( csp );

  cleanup_csp( csp );
  
  return( NULL );

} /* serve */



int
main( int argc, char *argv[] ) {

  char buf[BUFSIZ];
  char *p, *q;

  char *configfile = NULL;
  FILE *configfp = NULL;

  char *jarfile = NULL;

  char *logfile = NULL;
  int debuglevel = 0;

  int suppress_vanilla_wafer = 0;

  char *haddr = "127.0.0.1";	/* default binding to localhost */
  int   hport = 8000;
  int bfd;

  int include_stats = 0;

  int multi_threaded  = 1;

  pthread_t tid;
  pthread_attr_t tattr;


/*##### Insert test calls here !!! #####*/

  init_proxy_args( argc, argv );

  if( argc > 1 ) {
    configfile = argv[1];
  }

  if( configfile && !(configfp = fopen(configfile, "r")) ) {
    errlogf( ERN, "main(): can't open configuration file '%s'\n", configfile );
    exit( 1 );
  }

  if( configfp ) {

    int line_num = 0;

    while( fgets(buf, sizeof(buf), configfp) ) {

      char cmd[BUFSIZ];
      char arg[BUFSIZ];
      char tmp[BUFSIZ];

      line_num++;

      strcpy(tmp, buf);

      if( (p = strpbrk(tmp, "#\r\n")) )
        *p = '\0';

      p = tmp;

      /* leading skip whitespace */
      while(*p && ((*p == ' ') || (*p == '\t'))) p++;

      q = cmd;

      while( *p && (*p != ' ') && (*p != '\t') )
        *q++ = *p++;

      *q = '\0';

      while( *p && ((*p == ' ') || (*p == '\t')) )
        p++;

      strcpy(arg, p);

      p = arg + strlen( arg ) - 1;

      /* ignore trailing whitespace */
      while( *p && ((*p == ' ') || (*p == '\t')) )
        *p-- = '\0';

      if( *cmd == '\0' )
        continue;

      /* insure the command field is lower case */
      for( p = cmd; *p; p++ )
        *p = tolower( *p );

      savearg( cmd, arg );

      if( strcmp(cmd, "debug") == 0 ) {
        debuglevel |= atoi( arg );
        continue;
      }

      if( strcmp(cmd, "tinygif") == 0 ) {
        tinygif = atoi( arg );
        continue;
      }

      if( strcmp(cmd, "add-forwarded-header") == 0 ) {
        add_forwarded = 1;
        continue;
      }

      if( strcmp(cmd, "single-threaded") == 0 ) {
        multi_threaded = 0;
        continue;
      }

      if( strcmp(cmd, "suppress-vanilla-wafer") == 0 ) {
        suppress_vanilla_wafer = 1;
        continue;
      }

      if( strcmp(cmd, "wafer") == 0 ) {
        enlist( wafer_list, arg );
        continue;
      }

      if( strcmp(cmd, "add-header") == 0 ) {
        enlist( xtra_list,  arg );
        continue;
      }

      if( strcmp(cmd, "aclfile") == 0 ) {
        init_acl( arg );
        continue;
      }

      if( strcmp(cmd, "blockfile") == 0 ) {
        init_block( arg );
        continue;
      }

      if( strcmp(cmd, "cookiefile") == 0 ) {
        init_cookieblock( arg );
        continue;
      }

      if( strcmp(cmd, "forwardfile") == 0 ) {
        init_fwd( arg );
        continue;
      }

      if( strcmp(cmd, "logfile") == 0 ) {
        logfile = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "jarfile") == 0 ) {
        jarfile = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "listen-address") == 0 ) {
        haddr = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "user-agent") == 0 ) {
        uagent = strdup( arg );
        continue;
      }

      if( (strcmp(cmd, "referrer") == 0) || (strcmp(cmd, "referer") == 0) ) {
        referrer = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "from") == 0 ) {
        from = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "hide-console") == 0 ) {
        continue;	/* for compatibility with the real JunkBuster :) -gls */
      }

      if( strcmp(cmd, "passageword") == 0 ) {
        passageword = strdup( arg );
        continue;
      }

      if( strcmp(cmd, "include-stats") == 0 ) {
        include_stats = 1;
        continue;
      }

      if( strcmp(cmd, "suppress-blocklists") == 0 ) {
        suppress_message = strdup( arg );
        continue;
      }

     errlogf( ERR, "main(): unrecognized directive in configuration file "
                    "at line number %d:\n%s",
                    line_num, buf );
      exit( 1 );
    } /* while( fgets ) */

    fclose( configfp );

  } /* if( configfp ) */


  init_errlog( argv[0], logfile, debuglevel );


  if( jarfile ) {

    if( !(jar = fopen(jarfile, "a")) ) {
      errlogf( ERN, "main(): can't open jarfile: %s\n", jarfile );
      exit( 1 );
    }
    setbuf(jar, NULL);

    /* since we're logging cookies in a cookie jar,
     * if the user has not supplied any wafers,
     * and the user has not told us to suppress the vanilla wafer,
     * then send the vanilla wafer.
     */
    if( !(wafer_list->next) && !suppress_vanilla_wafer )
      enlist( wafer_list, VANILLA_WAFER );

  } /* if( jarfile ) */


#ifdef WIN_GCC
  if( winsock_init() != 0 ) {
    errlogf( ERR, "main(): winsock_init() failed\n" );
    exit( 1 );
  }
#endif


  if( haddr ) {

    if( (p = strchr(haddr, ':')) ) {
      *p++ = '\0';
      if( *p )
        hport = atoi( p );
    }

    if( hport <= 0 ) {
      *--p = ':';
      errlogf( ERR, "main(): invalid bind port spec %s\n", haddr );
      exit( 1 );
    }

    if( *haddr == '\0' )
      haddr = NULL;

  } /* if( haddr ) */

  errlogf( CON, "bind (%s, %d)\n", haddr ? haddr : "INADDR_ANY", hport );

  if( (bfd = bind_port(haddr, hport)) < 0 ) {
    errlogf( ERR, "main(): can't bind %s:%d\nThere may be another instance "
                  "of this or some other proxy running on port %d\n",
                  haddr ? haddr : "INADDR_ANY", hport, hport );
    errlogf( ERR, "main(): bind_port() returned with error code %d.\n", bfd );
    exit( 1 );
  } /* if( bfd = bind_port ) */


  end_proxy_args();


  /* If we're trying to write to a client that is gone we might get a SIGPIPE...
   * We don't want to get killed because of it, so there: 
   */
#ifndef WIN_GCC
  signal( SIGPIPE, SIG_IGN );
#endif


#ifdef SOL_SETCONC
  /* this is a hack to allow real thread concurrency on Solaris (GLS) */
  /* pthread_setconcurrency() does not exist on Linux (yet) !!! */
  if( pthread_setconcurrency(SOL_SETCONC) != 0 ) {
    errlogf( ERN, "main(): err. setting solaris thread concurrency\n" );
    exit( 1 );
  }
#endif /* SOL_SETCONC */


  /* Initialize the SmartCookie module */
  SmartCookieInit();

  /* init. the url, cookie and html encoding maps _before_ starting threaded
   * operation. All other public functions from urlcodec.h are thread-safe...
   */
  init_code_maps();

  /* init. the block stats module */
  init_stats( include_stats );


  /* init. detached thread attribute */
  pthread_attr_init( &tattr );
  pthread_attr_setdetachstate( &tattr, PTHREAD_CREATE_DETACHED );

  /* start site monitor thread */
/* temporarily disabled for windows testing 
  if( pthread_create(&tid, &tattr, site_monitor, NULL) != 0 ) {
    errlogf( ERN, "main(): could not start SiteMonitor thread\n" );
    exit( 1 );
  }
*/

  for( ; ; ) { /* service cycle */

    struct client_state *csp;

    /* create and initialize csp object */
    if( !(csp = new_csp()) ) {
      errlogf( ERN, "main(): failed to create csp object.\n" );
      exit( 1 );
    }

    /* accept client connection */
    errlogf( CON, "accept connection ... " );
    if( (csp->cfd = accept_connection(bfd, &(csp->ip_addr_str),
                                           &(csp->ip_addr_long))) < 0 ) {
      errlogf( ERR, "main(): accept failed\n" );
      cleanup_csp( csp );
      continue;
    }
    errlogf( CON, "OK\n" );

    /* serve client */
    if( multi_threaded ) {

      if( pthread_create(&tid, &tattr, serve, (void *)csp) != 0 ) {

        errlogf( ERN, "main(): can't create thread\n" );

        sprintf( buf , "SurfAgent: can't start thread: errno = %d", errno );
        write_and_log( csp->cfd, buf, strlen(buf) );

        close_socket( csp->cfd );

        cleanup_csp( csp );

        sleep( 5 );
        continue;
      }

    } else {	/* no thread */

      serve( csp );

    }

  } /* for( ; ; ) service cycle */

} /* main */
