/* This file is a component of the JunkBusterMT Proxy, based on original code from
 * the Internet Junkbuster Proxy.
 *
 * Copyright (C) 2000 Gabriel L. Somlo
 * Copyright (C) 1997 Anonymous Coders and Junkbusters Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see ../GPL.txt); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/************************************ csp.c ************************************
	Purpose: Housekeeping associated with the client_state structure
	Notes: None.
***/        

#include <unistd.h>
#include <stdlib.h>

#include "csp.h"		/* this also gets most of the other stuff...*/
#include "miscstr.h"
#include "bindconn.h"


/*------------------------------------------------------------------------*/
/* Create a new csp. All fields must be set to 0 or NULL, except the      */
/*  cfd and sfd file descriptors (set to -1), the accept_image_only       */
/*  field, set to 1 (TRUE) by default and wordvec, which is initialized)  */
/* Returns a pointer to the new structure on success, or NULL on failure. */
/*------------------------------------------------------------------------*/
struct client_state *
new_csp( void ) {

  struct client_state *csp;

  if( (csp = zalloc(sizeof(struct client_state))) ) {
    csp->cfd = csp->sfd = -1;
    csp->accept_image_only = 1;
  }

  return( csp );

} /* new_csp */


/*-----------------------------------------------------------------------*/
/* This cleanup routines must be separate, because chat() has multiple   */
/* exit points, and we can't concentrate all cleanup related things into */
/* a single point within chat... We'll have to fix this eventually...    */
/*-----------------------------------------------------------------------*/
void
cleanup_csp( struct client_state *csp ) {

  if( csp->cfd >= 0 )
    close_socket( csp->cfd );
  if( csp->sfd >= 0 )
    close_socket(csp->sfd);
  freez( csp->ip_addr_str );

  freez( csp->iob->buf );	/* replace with the clean_iob() function call */

  free_http_request( csp->http );
  if( csp->gw ) {	/* this should always be true... */
    freez( csp->gw->gateway_host );
    freez( csp->gw->forward_host );
    free( csp->gw );
  }
  destroy_list( csp->headers );

  destroy_list( csp->cookie_list );

  freez( csp->user_id );

  freez( csp->x_forwarded );


  freez( csp );

} /* cleanup_csp */
