/* This file is a component of the JunkBusterMT Proxy, based on original code from
 * the Internet Junkbuster Proxy.
 *
 * Copyright (C) 2000 Gabriel L. Somlo
 * Copyright (C) 1997 Anonymous Coders and Junkbusters Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see ../GPL.txt); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/********************************* filegard.h **********************************

    Purpose: Interface to the file guardians (acl, block, cookie, forward).

    Notes:   None.
**/

#ifndef FILEGARD_H
#define FILEGARD_H


#include "gnu_regex.h"	/* for regex_t */
#include "parsers.h"	/* for http_request */


#define COOKIE_MSK_ACC_SRV 4	/* mask for incoming cookies */
#define COOKIE_MSK_SND_CLI 2	/* mask for outgoing cookies */
#define COOKIE_MSK_SMART   1	/* mask for smart cookies */


/* this allows the proxy to permit/block access to any host and/or path */
struct url_spec {
	char  *spec;
	char  *domain;
	char  *dbuf;
	char **dvec;
	int    dcnt;
	int    toplevel;

	char *path;
	int   pathlen;
	int   port;

	regex_t *preg;
};

struct access_control_addr {
	unsigned long	addr;
	unsigned long	mask;
	unsigned long   port;
};

struct gateway {
	/* generic attributes */
        char *name;
        int (*conn)( struct gateway *gw, struct http_request *http,
                     long addr_long );
        int   type;

	/* domain specific attributes */
        char *gateway_host;
        int   gateway_port;

	char *forward_host;
	int   forward_port;
};

extern char *suppress_message;

extern void init_acl( char *aclfname );
extern int  block_acl( struct access_control_addr *src,
                       struct access_control_addr *dst );

extern void init_block( char *blockfname );
extern char *is_blocked_url( struct http_request *http );

extern void init_cookieblock( char *cookiefname );
extern int  cookie_url( struct http_request *http );

extern void init_fwd( char *forwardfname );
extern struct gateway *forward_url( struct http_request *http );

extern void savearg( char *cmd, char *arg );
extern void init_proxy_args( int argc, char **argv );
extern void end_proxy_args( void );
extern char *get_proxy_args( void );


#endif /* FILEGARD_H */
