/* This file is a component of the JunkBusterMT Proxy, based on original code from
 * the Internet Junkbuster Proxy.
 *
 * Copyright (C) 2000 Gabriel L. Somlo
 * Copyright (C) 1997 Anonymous Coders and Junkbusters Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see ../GPL.txt); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "gnu_regex.h"

#include "urlcodec.h"

#include "chat.h"	/* for various #defines */

#include "errlog.h"
#include "miscstr.h"

#include "filegard.h"

#include "parsers.h"

#include "csp.h"


char CMOVPERM[] =
  "HTTP/1.0 301 Moved Permanently\n"
  "Location: %s%s%s\n"
  "Content-Type: text/html\n\n"
  "<html>\n"
  "<head><title>301 Moved Permanently</title></head>\n"
  "<body><h1>Moved Permanently</h>\n"
  "The document is now <a href=\"%s%s%s\">here</a>.<p>\n"
  "</body></html>\n"
  ;


/* private type */
struct interceptors {
	char *str;
	char  len;
	char *(*interceptor)( struct http_request *http,
                              struct client_state *csp );
};


/* interceptor functions - used in struct interceptors intercept_patterns */
static char *show_proxy_args(   struct http_request *http,
                                struct client_state *csp );
static char *ij_blocked_url(    struct http_request *http,
                                struct client_state *csp );


static struct interceptors intercept_patterns[] = {
	{ "show-proxy-args",	15,	show_proxy_args		},
	{ "ij-blocked-url",	14,	ij_blocked_url		},

	{ NULL,			 0,	NULL			}
};



/* dsplit() takes a domain and returns a url_spec
 * structure populated with dbuf, dcnt and dvec.  the other fields
 * in the structure that is returned are zero.
 */
struct url_spec
dsplit( char *domain ) {

  struct url_spec ret;
  char *v[BUFSIZ];
  int size;
  char *p;

  memset( &ret, '\0', sizeof(struct url_spec) );

  if( (p = strrchr(domain, '.')) && (*(++p) == '\0') )
      ret.toplevel = 1;

  ret.dbuf = strdup( domain );

  /* map to lower case */
  for( p = ret.dbuf; *p ; p++ )
    *p = tolower( *p );

  /* split the domain name into components */
  ret.dcnt = ssplit( ret.dbuf, ".", v, SZ(v), 1, 1 );

  if( ret.dcnt <= 0 ) {
    memset( &ret, '\0', sizeof(struct url_spec) );
    return( ret );
  }

  /* save a copy of the pointers in dvec */
  size = ret.dcnt * sizeof( char * );

  if( (ret.dvec = malloc( size )) ) {
    memcpy( ret.dvec, v, size );
  }

  return( ret );

} /* dsplit */

/* the "pattern" is a domain that may contain a '*' as a wildcard.
 * the "fqdn" is the domain name against which the patterns are compared.
 *
 * domaincmp("a.b.c" , "a.b.c")	=> 0 (MATCH)
 * domaincmp("a*.b.c", "a.b.c")	=> 0 (MATCH)
 * domaincmp("b.c"   , "a.b.c")	=> 0 (MATCH)
 * domaincmp(""      , "a.b.c")	=> 0 (MATCH)
 */
int
domaincmp( struct url_spec *pattern, struct url_spec *fqdn ) {

  char **pv, **fv;	/* vectors  */
  int    pn,   fn;	/* counters */
  char  *p,   *f;	/* chars    */
	
  pv = pattern->dvec;
  pn = pattern->dcnt;

  fv = fqdn->dvec;
  fn = fqdn->dcnt;

  while( (pn > 0) && (fn > 0) ) {

    p = pv[--pn];
    f = fv[--fn];

    for( ; *p && *f && (*p == tolower(*f)); p++, f++ );

    if( (*p != tolower(*f)) && (*p != '*') )
      return( 1 );
  }

  if( pn > 0 )
    return( 1 );

  return(0);
} /* domaincmp */



/******************************************************************************/

/* the following functions are invoked from chat() to catch url's that are
   either fake and thus need to be intercepted by JunkBuster, or there's
   something else wrong with them (i.e., on the blocklist, etc.)
 */


/* checks if the URL matches something on the blocklist */
char *
block_url( struct http_request *http ) {

  char *templ_str =
    "HTTP/1.0 202 Request for blocked URL\n"
    "Pragma: no-cache\n"
    "Last-Modified: Thu Jul 31, 1997 07:42:22 pm GMT\n"
    "Expires:       Thu Jul 31, 1997 07:42:22 pm GMT\n"
    "Content-Type: text/html\n\n"
    "<html>\n"
    "<head>\n"
    "<title>Internet Junkbuster: Request for blocked URL</title>\n"
    "</head>\n"
    BODY
    "<center><h1>" BANNER "</h1></center>\n"
    BANNER " is blocking this page because it "
    "<a href=http://internet.junkbuster.com/ij-blocked-url?%s+%s+%s>matches</a>"
    " one of the patterns in the blocklist.\n"
    "<p>Click <a href=\"%s%s\">here</a> to circumvent the block and view the "
    "page anyway (may require more than one click).\n"
    "</body>\n"
    "</html>\n"
    ;

  char *bspec;
  char *p;
  char *hostport, *path, *spec;
  int n;

  if( (bspec = is_blocked_url(http)) ) {

    hostport = url_encode( http->hostport );
    path     = url_encode( http->path );
    spec     = url_encode( bspec );

    n  = strlen( templ_str );
    n += strlen( hostport );
    n += strlen( path );
    n += strlen( spec );
    n += strlen( http->path );
    n += strlen( passageword );

    p = malloc( n );

    sprintf( p, templ_str, hostport, path, spec, http->path, passageword );

    /* objects were allocated by url_encode() */
    freez( hostport );
    freez( path );
    freez( spec );

    /* object was allocated by is_blocked_url() */
    free( bspec );	/* we know for sure it's NOT NULL */

    return( p );
  }

  return( NULL );

} /* block-url */


/* this checks if the url is fake and needs to be intercepted by JunkBuster */
/* intercept_url() checks the URL `basename' against a list of URLs
 * to snarf.  If it matches, it calls the associated function which
 * returns an HTML page to send back to the client.
 */
char *
intercept_url( struct client_state *csp ) {

  struct http_request *http;
  char *basename;
  struct interceptors *v;

  http = csp->http;
  basename = strrchr( http->path, '/' );

  if( !basename )
    return( NULL );

  ++basename;	/* first char past the last slash */

  if( *basename != '\0' )
    for( v = intercept_patterns; v->str; v++ )
      if( strncmp(basename, v->str, v->len) == 0 )
        return( (v->interceptor)(http, csp) );

  return( NULL );

} /* intercept_url */

/******************************************************************************/

/* interceptor functions - used in struct interceptors intercept_patterns */


/* display the options used when JunkBuster was started */
static char *
show_proxy_args( struct http_request *http, struct client_state *csp ) {

  /* forward the call to the routine in the filegard module */
  return( get_proxy_args() );

} /* show_proxy_args */


/* This function offers an explanation as to why a site was blocked
 * (when the user clicks the link offerred on the block notification)
 */
static char *
ij_blocked_url( struct http_request *http, struct client_state *csp ) {

  char *templ_str =
    "HTTP/1.0 200 OK\r\n"
    "Pragma: no-cache\n"
    "Last-Modified: Thu Jul 31, 1997 07:42:22 pm GMT\n"
    "Expires:       Thu Jul 31, 1997 07:42:22 pm GMT\n"
    "Content-Type: text/html\n\n"
    "<html>\n"
    "<head>\n"
    "<title>Internet Junkbuster: Request for blocked URL</title>\n"
    "</head>\n"
    BODY
    "<center><h1>" BANNER "</h1></center>\n"
    "The " BANNER " Proxy "
    "<A href=\"http://www.junkbuster.com\"> (http://www.junkbuster.com) </A> "
    "intercepted the request for %s%s because the URL matches the following "
    "pattern in the blockfile: %s\n"
    "</body>\n"
    "</html>\n"
    ;

  int n;
  char *hostport, *path, *pattern, *p, *v[9];

  if( (n = ssplit(http->path, "?+", v, SZ(v), 1, 1)) == 4 ) {
    hostport = url_decode( v[1] );
    path     = url_decode( v[2] );
    pattern  = url_decode( v[3] );
  } else {
    hostport = strdup( "undefined_host" );
    path     = strdup( "/undefined_path" );
    pattern  = strdup( "undefined_pattern" );
  }

  n  = strlen(templ_str );
  n += strlen(hostport  );
  n += strlen(path      );
  n += strlen(pattern   );

  if( (p = malloc(n)) )
    sprintf( p, templ_str, hostport, path, pattern );

  freez( hostport );
  freez( path     );
  freez( pattern  );

  return( p );

} /* ij_blocked_url */
